@@ -106,6 +106,7 @@ GnomeICUPref gnomeicu_prefs[] = {
   { PREFS_GNOMEICU_SPELLCHECK, "gicu_spellcheck", PREF_BOOLEAN, NULL },
   { PREFS_GNOMEICU_DEFAULT_CHAT_MODE, "gicu_default_chat_mode", PREF_BOOLEAN, NULL },
   { PREFS_GNOMEICU_CHARSET_FALLBACK, "gicu_charset_fallback_combo", PREF_COMBO, &lang_combo_box },
+  { PREFS_GNOMEICU_ENTER_SENDS, "gicu_enter_sends_radio", PREF_BOOLEAN, NULL},
   { PREFS_GNOMEICU_ICON_THEMES, "gicu_icon_theme_treeview", PREF_CUSTOM, &icon_theme_treeview },
   { PREFS_GNOMEICU_EMOTICON_THEMES, "gicu_emoticon_theme_treeview", PREF_CUSTOM, &emoticon_theme_treeview },
 
@@ -367,6 +368,17 @@ preferences_set_color (const gchar *gconf_key, const GdkColor *data)
   g_free (color_string);
 }
 
+void 
+preferences_unset(const gchar *gconf_key)
+{
+  g_return_if_fail (gconf_key);
+  
+  if (!prefs_init)
+    preferences_init ();
+
+  gconf_client_unset (client, gconf_key, NULL);
+}
+
 void
 preferences_notify_cb (GConfClient *client, guint cnxn_id,
                        GConfEntry *entry, gpointer func)
@@ -391,7 +403,7 @@ preferences_set_pref_widget_relation (const GnomeICUPrefData *pd, gint i)
     return;
 
   if (GTK_IS_TOGGLE_BUTTON (pd[i].widget)) {
-    active = (gboolean)pd[i].value;
+    active = (gboolean)GPOINTER_TO_INT(pd[i].value);
   }
 
   if (*pr == PR_SENSITIVITY_BEGIN_INVERSE)
@@ -458,13 +470,13 @@ preferences_spinner_value_changed_cb (GtkSpinButton *widget,
 
   value = gtk_spin_button_get_value_as_int (widget);
 
-  pd->value = (gpointer)value;
+  pd->value = GINT_TO_POINTER(value);
 
   preferences_set_int (pd->key, value);
 }
 
 static void
-preferences_toggle_button_clicked_cb (GtkToggleButton *widget,
+preferences_toggle_button_toggled_cb (GtkToggleButton *widget,
                                       GnomeICUPrefData *pd)
 {
   gboolean b;
@@ -472,7 +484,7 @@ preferences_toggle_button_clicked_cb (GtkToggleButton *widget,
   b = gtk_toggle_button_get_active (widget);
 
   preferences_set_bool (pd->key, b);
-  pd->value = (gpointer)b;
+  pd->value = GINT_TO_POINTER(b);
 
   preferences_set_pref_widget_relation (pd, 0);
 }
@@ -537,8 +549,8 @@ preferences_connect_signal_spin_button (GnomeICUPrefData *pd)
 static void
 preferences_connect_signal_toggle_button (GnomeICUPrefData *pd)
 {
-  g_signal_connect (G_OBJECT (pd->widget), "clicked",
-                    G_CALLBACK (preferences_toggle_button_clicked_cb),
+  g_signal_connect (G_OBJECT (pd->widget), "toggled",
+                    G_CALLBACK (preferences_toggle_button_toggled_cb),
                     (gpointer)pd);
 }
 
@@ -642,12 +654,23 @@ init_prefs (GladeXML *gxml, const GnomeICUPref *prefs)
 
     switch (prefs[i].type) {
     case PREF_UINT:
-      pd[i].value = (gpointer)gconf_client_get_int (client, prefs[i].gconf_key,
-                                                    NULL);
+      pd[i].value = GINT_TO_POINTER(gconf_client_get_int (client, 
+							  prefs[i].gconf_key,
+							  NULL));
       break;
     case PREF_BOOLEAN:
-      pd[i].value = (gpointer)gconf_client_get_bool (client, prefs[i].gconf_key,
-                                                     NULL);
+      pd[i].value = GINT_TO_POINTER(gconf_client_get_bool (client, 
+							   prefs[i].gconf_key,
+							   NULL));
+      {
+	gchar *name = g_strdup_printf("%s_other", prefs[i].widget_name);
+	GtkWidget *tmpwidget;
+	tmpwidget = glade_xml_get_widget (gxml, name);
+	if (tmpwidget) {
+	  g_object_set_data(G_OBJECT(pd[i].widget), "other", tmpwidget);
+	}
+      g_free(name);
+      }
       break;
     case PREF_STRING:
     case PREF_COLOR:
@@ -900,7 +923,8 @@ set_spin_button (const GnomeICUPrefData *pd)
 {
   switch (pd->type) {
   case PREF_UINT:
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON (pd->widget), (guint)(pd->value));
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON (pd->widget), 
+			       GPOINTER_TO_UINT(pd->value));
     break;
   }
 }
@@ -908,10 +932,18 @@ set_spin_button (const GnomeICUPrefData *pd)
 void
 set_toggle_button (const GnomeICUPrefData *pd)
 {
+  GtkWidget *widget;
+
   switch (pd->type) {
   case PREF_BOOLEAN:
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pd->widget),
-                                  (gboolean)(pd->value));
+                                  GPOINTER_TO_INT(pd->value));
+    widget = (GtkWidget *) g_object_get_data(G_OBJECT(pd->widget), "other");
+    if (widget)
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
+				    ! GPOINTER_TO_INT(pd->value));
+      
+    g_print("toggle_set_active key:%s (%d)\n", pd->key, pd->value);
     break;
   }
 }
@@ -928,7 +960,7 @@ set_editable (const GnomeICUPrefData *pd)
     gtk_entry_set_text (GTK_ENTRY (pd->widget), pd->value);
     break;
   case PREF_UINT:
-    str = g_strdup_printf ("%d", (guint)(pd->value));
+    str = g_strdup_printf ("%d", GPOINTER_TO_INT(pd->value));
     gtk_entry_set_text (GTK_ENTRY (pd->widget), str);
     g_free (str);
     break;
