@@ -4,24 +4,33 @@
  GnomeICU
 *********************************/
 
+#include "tcp.h"
 #include "common.h"
-#include "applet.h"
-#include "dirbrowser.h"
 #include "gnomeicu.h"
+#include "gnomeicu-spinner.h"
 #include "gtkfunc.h"
 #include "history.h"
 #include "icons.h"
-#include "kanji_conv.h"
 #include "packetprint.h"
 #include "response.h"
 #include "sendcontact.h"
 #include "showlist.h"
-#include "tcp.h"
 #include "util.h"
 #include "v7send.h"
 #include "v7base.h"
-#include "rus_conv.h"
 
+#include <gtk/gtk.h>
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <fcntl.h>
+#include <netinet/in.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
+#include <sys/types.h>
+#include <sys/socket.h>
 gint min_tcp_port = 4000;
 gint max_tcp_port = 5000;
 
@@ -38,8 +47,11 @@ static unsigned char client_check_data[] = {
   "ICQ Service and Information may\0"
 };
 
+enum {
+	ICQ_RESPONSE_SERVER,
+	ICQ_RESPONSE_RETRY
+};
 
-#define TRACE_FUNCTION 1
 
 /*** Local function declarations ***/
 static void tcp_encrypt ( guchar *data, int size, int version);
@@ -65,10 +77,6 @@ static int TCPFinishConnection( GIOChannel *source, GIOCondition cond, gpointer
 static int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data );
 static int TCPTimeout( tcp_message *m );
 
-static void timeout_clicked( GnomeDialog *dialog, gint button, tcp_message *m );
-
-
-GSList *pending_xfers = 0;
 
 /*** Global functions ***/
 
@@ -88,7 +96,7 @@ tcp_message *tcp_prepare_message( CONTACT_PTR contact, BYTE *data,
   m->seq = seq;
   m->text = text;
   m->data = g_memdup(data, Chars_2_Word(data)+2);
-  m->uin = contact->uin;
+  m->uin = g_strdup (contact->uin);
 
   contact->tcp_msg_queue = g_slist_append( contact->tcp_msg_queue, m );
 
@@ -110,8 +118,7 @@ gboolean TCPSendPacket(UIN_T uin,
 #endif
 
   contact = Find_User( uin );
-  if( contact == NULL || kontakt->force_override ||
-      kontakt->has_direct_connect != YES)
+  if( contact == NULL || kontakt->has_direct_connect != YES)
     return FALSE;
   
   
@@ -128,10 +135,10 @@ gboolean TCPSendPacket(UIN_T uin,
   switch (kontakt->used_version) {
   case 2:    
   case 3:
-    add_nontlv_dw_le(tlvs, our_info->uin);
+    add_nontlv_dw_le(tlvs, atoi (our_info->uin));
     add_nontlv_w_le (tlvs, kontakt->used_version);
     add_nontlv_dw_le(tlvs, command);
-    add_nontlv_dw_le(tlvs, our_info->uin);
+    add_nontlv_dw_le(tlvs, atoi (our_info->uin));
     add_nontlv_w_le (tlvs, subcommand);
     add_nontlv_lnts (tlvs, message);
     add_nontlv_dw_le(tlvs, our_ip);
@@ -156,11 +163,11 @@ gboolean TCPSendPacket(UIN_T uin,
     break;
   case 4:
   case 5:
-    add_nontlv_dw_le(tlvs, our_info->uin);
+    add_nontlv_dw_le(tlvs, atoi (our_info->uin));
     add_nontlv_w_le (tlvs, kontakt->used_version);
     add_nontlv_dw_le(tlvs, 0);               /* CHECKSUM */
     add_nontlv_dw_le(tlvs, command);
-    add_nontlv_dw_le(tlvs, our_info->uin);
+    add_nontlv_dw_le(tlvs, atoi (our_info->uin));
     add_nontlv_w_le (tlvs, subcommand);
     add_nontlv_lnts (tlvs, message);
     add_nontlv_dw_le(tlvs, our_ip);
@@ -180,6 +187,7 @@ gboolean TCPSendPacket(UIN_T uin,
     break;
   case 6:
   case 7:
+  case 8:
     add_nontlv_dw_le(tlvs, 0); /* CHECKSUM */
     add_nontlv_w_le (tlvs, command); 
     add_nontlv_w_le (tlvs, 0x0E);  /* always 0x0E (copied from licq) */
@@ -197,6 +205,8 @@ gboolean TCPSendPacket(UIN_T uin,
     if (data) 
       add_nontlv    (tlvs, data, datalen);
     break;
+	default:
+	 g_warning("Unknown version used by contact for sending packet: %d\n", kontakt->used_version);
   };
 
   tcp_encrypt(tlvs->beg+2, tlvs->len-2, kontakt->used_version); /* checksum */
@@ -233,7 +243,7 @@ gboolean TCPSendPacket(UIN_T uin,
   else
     return FALSE;
   
-  animate_on();
+  gnomeicu_spinner_start();
 
   return TRUE;
 }
@@ -267,6 +277,7 @@ void tcp_encrypt ( guchar *data, int datalen, int version)
     break;
   case 6:
   case 7:
+  case 8:
   default:
     offset = 0;
   }
@@ -334,6 +345,7 @@ gboolean tcp_decrypt(guchar *data, int datalen, int version)
     break;
   case 6:
   case 7:
+  case 8:
   default:
     offset = 0;
   }
@@ -513,8 +525,9 @@ int TCPSendURL( UIN_T uin, gchar *msg )
                        0, NULL, 0, msg, 0 ,0, TRUE);
 }
 
-int TCPAcceptIncoming( gpointer data, int sock, GdkInputCondition cond )
+int TCPAcceptIncoming( GIOChannel *iochan, GIOCondition cond, gpointer data )
 {
+  int sock = g_io_channel_unix_get_fd(iochan);
   struct sockaddr_in addr;
   int size = sizeof( struct sockaddr_in );
   int new_sock;
@@ -575,7 +588,7 @@ int TCPRetrieveAwayMessage( GSList *contact, gpointer data )
 int TCPAcceptFile( XferInfo *xfer )
 {
   TLVstack *tlvs;
-  int returncode;
+  int returncode=0;
 
 #ifdef TRACE_FUNCTION
   g_print( "TCPAcceptFile\n" );
@@ -591,13 +604,16 @@ int TCPAcceptFile( XferInfo *xfer )
   add_nontlv_dw_le(tlvs, xfer->port);
 
   
-  if (toggles->auto_accept_xfers) {
-    xfer->auto_save_path = g_strdup(toggles->auto_accept_path);
+  if (preferences_get_bool (PREFS_GNOMEICU_AUTO_ACCEPT_FILE)) {
+    if (xfer->auto_save_path)
+      g_free (xfer->auto_save_path);
+
+    xfer->auto_save_path = preferences_get_string (PREFS_GNOMEICU_DOWNLOAD_PATH);
   }
   
-  returncode =  TCPAckPacket(xfer->remote_contact->uin,ICQ_CMDxTCP_FILE,
-                             tlvs->beg, tlvs->len, ICQ_ACKxTCP_ONLINE,
-                             xfer->seq);
+  //  returncode =  TCPAckPacket(xfer->remote_contact->uin,ICQ_CMDxTCP_FILE,
+  //                             tlvs->beg, tlvs->len, ICQ_ACKxTCP_ONLINE,
+  //                             xfer->seq);
 
   free_tlvstack(tlvs);
 
@@ -608,17 +624,18 @@ int TCPAcceptFile( XferInfo *xfer )
 int TCPRefuseFile( XferInfo *xfer )
 {
 
-  int returncode;
+  int returncode=0;
   
 #ifdef TRACE_FUNCTION
   g_print( "TCPRefuseFile\n" );
 #endif
   
-  pending_xfers = g_slist_remove(pending_xfers, xfer);
+  xfer->remote_contact->file_xfers = g_slist_remove(xfer->remote_contact->file_xfers, xfer);
+
   
-  returncode = TCPSendPacket(xfer->remote_contact->uin, ICQ_CMDxTCP_CANCEL,
-                             ICQ_CMDxTCP_FILE, 0, 0, NULL, 0, "",
-                             ICQ_ACKxTCP_REFUSE, xfer->seq, FALSE);
+  //  returncode = TCPSendPacket(xfer->remote_contact->uin, ICQ_CMDxTCP_CANCEL,
+			     //                             ICQ_CMDxTCP_FILE, 0, 0, NULL, 0, "",
+			     //                             ICQ_ACKxTCP_REFUSE, xfer->seq, FALSE);
   g_free(xfer);
 
   return returncode;
@@ -632,14 +649,13 @@ int TCPSendFileRequest( UIN_T uin, const gchar *msg, GSList *files )
   struct stat file_stat;
   gchar *nopathfile = 0;
 
-  struct file_struct *fileinfo;
   XferInfo *xferinfo;
 
   GSList *file;
   GSList *contact;
   TLVstack *tlvs;
 
-  int returncode;
+  int returncode=0;
 
 #ifdef TRACE_FUNCTION
   g_print( "TCPSendFileRequest\n" );
@@ -649,13 +665,9 @@ int TCPSendFileRequest( UIN_T uin, const gchar *msg, GSList *files )
   if( !contact  || kontakt->has_direct_connect != YES)
     return FALSE;
 
-  xferinfo = g_malloc0(sizeof(XferInfo));
+  xferinfo = ft_new(kontakt, XFER_DIRECTION_SENDING, NULL, 0);
 
-  xferinfo->seq = kontakt->tcp_seq--;
-  xferinfo->remote_contact = kontakt;
-
-  xferinfo->direction = XFER_DIRECTION_SENDING;
-  xferinfo->status = XFER_STATUS_PENDING;
+  //  xferinfo->seq = kontakt->tcp_seq--;
 
   for (file = files;file;file = file->next) {
     nopathfile = ((gchar*)file->data);
@@ -666,33 +678,24 @@ int TCPSendFileRequest( UIN_T uin, const gchar *msg, GSList *files )
         break;
       }
 
-    if( strlen( nopathfile ) == 0 )
+    if( !nopathfile || nopathfile[0] == '\0' )
       return FALSE;
 
     if( stat( (gchar*)file->data, &file_stat ) == -1 )
       return FALSE;
 
-    fileinfo = g_malloc0(sizeof(struct file_struct));
-
-    fileinfo->full_filename = g_strdup(((gchar*)file->data));
-    fileinfo->short_filename = g_strdup(nopathfile);
-
-    fileinfo->total_bytes = file_stat.st_size;
-    xferinfo->total_bytes += file_stat.st_size;
-
-    xferinfo->file_queue = g_slist_append(xferinfo->file_queue, fileinfo);
+    ft_addfile(xferinfo, g_strdup(((gchar*)file->data)), g_strdup(nopathfile),
+	       file_stat.st_size);
   }
 
   if (g_slist_length(xferinfo->file_queue) == 1)
     xferinfo->filename = g_strdup(nopathfile);
   else {
-    GString *buf = g_string_new("");
-    g_string_sprintf(buf, "%u files", g_slist_length(xferinfo->file_queue));
-    xferinfo->filename = g_strdup(buf->str);
-    g_string_free(buf, TRUE);
+    char buf[15];
+    snprintf(buf, sizeof (buf), "%u files", g_slist_length(xferinfo->file_queue));
+    xferinfo->filename = g_strdup(buf);
   }
 
-  pending_xfers = g_slist_append(pending_xfers, xferinfo);
 
   tlvs = new_tlvstack("\0\0\0\0", 4);
   add_nontlv_lnts(tlvs, xferinfo->filename);
@@ -700,9 +703,9 @@ int TCPSendFileRequest( UIN_T uin, const gchar *msg, GSList *files )
   add_nontlv_dw_le(tlvs, 0); /* licq uses 0, old gnomeicu used 0x0100 */
 
         
-  returncode = TCPSendPacket(uin, ICQ_CMDxTCP_START, ICQ_CMDxTCP_FILE,
+  /*  returncode = TCPSendPacket(uin, ICQ_CMDxTCP_START, ICQ_CMDxTCP_FILE,
                              0, 0, tlvs->beg, tlvs->len, msg, 0,
-                             xferinfo->seq, TRUE);
+                             xferinfo->seq, TRUE); */
   free_tlvstack(tlvs);
 
   if (!returncode)
@@ -731,7 +734,7 @@ void TCPSendContactList( GtkWidget *widget, gpointer data )
   unsigned short intsize;
   guchar *msg;
   BYTE *buffer;
-  UIN_T uin = GPOINTER_TO_INT( gtk_object_get_data( GTK_OBJECT( widget ), "uin" ) );
+  UIN_T uin = g_object_get_data (G_OBJECT (widget ), "uin" );
 
   typedef struct
   {
@@ -777,11 +780,11 @@ void TCPSendContactList( GtkWidget *widget, gpointer data )
       msg = g_strconcat( msg, nick, "\xFE", NULL );
     }
 
-  DW_2_Chars( packet.head.uin_a, our_info->uin );
+  DW_2_Chars( packet.head.uin_a, atoi (our_info->uin) );
   Word_2_Chars( packet.head.version, 0x0003 );
   Word_2_Chars( packet.head.cmd, ICQ_CMDxTCP_START );
   Word_2_Chars( packet.head.zero, 0x0000 );
-  DW_2_Chars( packet.head.uin_b, our_info->uin );
+  DW_2_Chars( packet.head.uin_b, atoi (our_info->uin) );
   Word_2_Chars( packet.head.command, ICQ_CMDxTCP_CONT_LIST );
   Word_2_Chars( packet.head.msg_length, ( strlen( msg ) + 1 ) );
 
@@ -828,7 +831,7 @@ void TCPSendContactList( GtkWidget *widget, gpointer data )
 
   if( sock == -2 ) /* We're waiting for a connection */
     {
-      animate_on();
+      gnomeicu_spinner_start();
       return;
     }
 
@@ -837,7 +840,7 @@ void TCPSendContactList( GtkWidget *widget, gpointer data )
   else
     return;
 
-  animate_on();
+  gnomeicu_spinner_start();
   return;
 }
 
@@ -850,7 +853,7 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
   int sock;
   int version;
   GSList *contact;
-  guint bytes_read;
+  gsize bytes_read;
 	
 #ifdef TRACE_FUNCTION
   g_print( "TCPInitChannel(" );
@@ -874,8 +877,8 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
   if( bytes_read == 1 ) {
     bytes_read = 0;
     while( !bytes_read )
-      g_io_channel_read( source, (gchar*)&packet_size + 1,
-                         1, &bytes_read );
+      g_io_channel_read_chars( source, (gchar*)&packet_size + 1,
+			       1, &bytes_read , NULL);
   }
   
 
@@ -891,9 +894,9 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
   version = packet[1];
 
   if (version <= 5)
-    uin = Chars_2_DW (packet + 9);
+    uin = g_strdup_printf ("%d", Chars_2_DW (packet + 9));
   else
-    uin = Chars_2_DW (packet + 15);
+    uin = g_strdup_printf ("%d", Chars_2_DW (packet + 15));
 
   contact = Find_User( uin );  
 
@@ -917,8 +920,8 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
 
   
   if (version >= 6) {
-    TCPSendHandShakeAck(contact, kontakt->sok);
-    TCPSendHandShake(contact, Chars_2_DW(packet+32), kontakt->sok);
+    TCPSendHandShakeAck(kontakt->sok);
+    TCPSendHandShake(kontakt, Chars_2_DW(packet+32), kontakt->sok);
 
     kontakt->have_tcp_connection = INCOMING;
   }
@@ -927,6 +930,7 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
 
   g_free( data );
   g_free( packet );
+  g_free (uin);
   return FALSE;
 
 }
@@ -971,7 +975,9 @@ int TCPAckPacket(UIN_T uin, int subcommand, const gchar *data, int datalen, int
     return TCPSendPacket(uin, ICQ_CMDxTCP_ACK, subcommand, 0, 0, data,
                          datalen, "", status, seq, FALSE);
   else {
-    msg = kanji_conv_auto(Away_Message, KANJI_SJIS);
+    msg = convert_from_utf8(Away_Message);
+
+    
     returncode = TCPSendPacket(uin, ICQ_CMDxTCP_ACK, subcommand, 0, 0, data,
                                datalen, msg, status, seq, FALSE);
     g_free(msg);
@@ -982,8 +988,6 @@ int TCPAckPacket(UIN_T uin, int subcommand, const gchar *data, int datalen, int
 void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
 {
   BYTE *here;
-  GSList *xfer;
-
 
   WORD command = 0;
   WORD subcmd = 0;
@@ -994,26 +998,18 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
   WORD status = 0;
   WORD msgtype;
 
-  struct {
-    WORD name_len;
-    char *name;
-    WORD revport;
-    DWORD port;
-  } tcp_chat;
-
-  struct {
+struct {
     WORD revport;
     WORD name_len;
     char *name;
     DWORD size;
     WORD port;
-  } tcp_file;
+} tcp_file;
 
   gchar *desc;
   GSList *messages;
   tcp_message *m = NULL;
   GSList *contact = NULL;
-  ChatContact *ccontact;
 
   gchar *countstr, *tmp = NULL;
   ContactPair *cpair;
@@ -1044,7 +1040,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
   if (kontakt->have_tcp_connection == OUTGOING2) {
     
     /* Send hack for the hello packet*/
-    TCPSendHandShakeAck(contact, kontakt->sok);
+    TCPSendHandShakeAck(kontakt->sok);
     kontakt->have_tcp_connection = YES;
     TCPClearQueue(contact);
     return;
@@ -1142,6 +1138,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
     break;
   case 6:
   case 7:
+  case 8:
     here += 4; /* checksum(4), */
     
     command = Chars_2_Word(here);
@@ -1170,25 +1167,12 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
     here += message_length;
     
     break;
+	default:
+	 g_warning("Unknown version used by contact for processing packet: %d\n", kontakt->used_version);
   }
 
 
-  if (subcmd == ICQ_CMDxTCP_CHAT ) {
-    tcp_chat.name_len = Chars_2_Word( here );
-    here += 2;
-    if (tcp_chat.name_len  > 0) {
-      tcp_chat.name = g_memdup(here, tcp_chat.name_len);
-    } else
-      here += 1;
-    
-    here += tcp_chat.name_len;
-    tcp_chat.revport = Chars_2_DW( here );
-    here += 4;
-    tcp_chat.port = Chars_2_DW( here );
-    here += 4;
-  }
-  
-  else if ( subcmd == ICQ_CMDxTCP_FILE ) {
+  if ( subcmd == ICQ_CMDxTCP_FILE ) {
     tcp_file.revport = Chars_2_DW( here );
     here += 4;
     tcp_file.name_len = Chars_2_Word( here );
@@ -1261,13 +1245,14 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
         return;
 
       for (i=0; i < count; i++) {
-        cpair = g_malloc(sizeof(ContactPair));
+        cpair = g_new0(ContactPair, 1);
 
         cpair->textuin = tmp;
         tmp = strchr( tmp, '\xFE' );
         if ( tmp == NULL )
           return; /* Bad Packet */
-        *tmp = '\0';            cpair->textuin = g_strdup(cpair->textuin);
+        *tmp = '\0';            
+		cpair->textuin = g_strdup(cpair->textuin);
         tmp++;
 
         cpair->nick = tmp;
@@ -1287,19 +1272,11 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
       TCPAckPacket( uin, subcmd, NULL, 0, 0, sequence);
       break;
 
-    case ICQ_CMDxTCP_CHAT:
-      packet_print( packet, packet_length,
-                    PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "CHAT REQUEST" );
-      ccontact = Do_Chat( tcp_chat.name_len, message, uin, sequence, tcp_chat.name, tcp_chat.port );
-      if( Current_Status == STATUS_FREE_CHAT )
-        TCPAcceptChat( sock, ccontact);
-      break;
-				
     case ICQ_CMDxTCP_FILE:
       packet_print( packet, packet_length,
                     PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "FILE REQUEST" );
 			
-      Do_File( message, uin, sequence, tcp_file.name, tcp_file.size );
+      //      Do_File( message, uin, sequence, tcp_file.name, tcp_file.size );
       break;
 
     default:
@@ -1322,7 +1299,8 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
           subcmd == ICQ_CMDxTCP_CONT_LIST ) {
         packet_print( packet, packet_length,
                       PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "MSG/URL/CONT LIST ACK" );
-        animate_off();
+
+	gnomeicu_spinner_stop();
 
         messages = kontakt->tcp_msg_queue;
         while( messages != NULL ) {
@@ -1376,19 +1354,11 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
       }
       break;
 
-    case ICQ_CMDxTCP_CHAT:
-      packet_print( packet, packet_length,
-                    PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "ACK CHAT REQUEST" );
-      if( tcp_chat.port > 0 )
-        TCPConnectChat( tcp_chat.port, uin, FALSE );
-      break;
-
     case ICQ_CMDxTCP_FILE:
       packet_print( packet, packet_length,
                     PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "ACK FILE REQUEST" );
-      for(xfer = pending_xfers;xfer;xfer = xfer->next)
-        if ( ( ((XferInfo*)xfer->data)->remote_contact->uin == uin) &&
-             ( ((XferInfo*)xfer->data)->seq == sequence) &&
+      /*      for(xfer = kontakt->file_xfers;xfer;xfer = xfer->next)
+        if ( ( ((XferInfo*)xfer->data)->seq == sequence) && 
              ( ((XferInfo*)xfer->data)->direction == XFER_DIRECTION_SENDING))
           break;
 
@@ -1396,7 +1366,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
         
         XferInfo *transfer = xfer->data;
         
-        pending_xfers = g_slist_remove(pending_xfers, transfer);
+        kontakt->file_xfers = g_slist_remove(kontakt->file_xfers, transfer);
         
         if( tcp_file.port > 0 ) {
           transfer->port = tcp_file.port;
@@ -1404,7 +1374,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
             ft_cancel_transfer(0, transfer);
         }
       }
-
+      */
       break;
 
     default:
@@ -1415,68 +1385,26 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
   case ICQ_CMDxTCP_CANCEL:
     switch ( subcmd ) {
 
-      ChatSession *csession;
-
-    case ICQ_CMDxTCP_CHAT:
-      packet_print( packet, packet_length,
-                    PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "CHAT REQ CANCEL" );
-      gnome_ok_dialog( _("Chat Request Refused") );
-					
-      if (g_list_first(kontakt->chat_requests))
-        {
-          csession = (ChatSession *) g_list_first(kontakt->chat_requests)->data;
-          kontakt->chat_requests = g_list_remove(kontakt->chat_requests, csession);
-
-          if (csession->members == NULL && csession->chatwindow == NULL) {
-            Sessions = g_slist_remove (Sessions, csession);
-            g_free(csession);
-          }
-        }
-      else
-        {
-          GSList *temp_storedmsg;
-					
-          temp_storedmsg = kontakt->stored_messages;
-          while (temp_storedmsg != NULL)
-            {
-              if (((STORED_MESSAGE_PTR) temp_storedmsg->data)->type == MESSAGE_CHAT_REQ)
-                break;
-              temp_storedmsg = temp_storedmsg->next;
-            }
-
-          if (temp_storedmsg != NULL)
-            {
-						
-              g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->message);
-              g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->chatsessionname);
-              g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->data);
-              g_free(temp_storedmsg->data);
-              kontakt->stored_messages = g_slist_remove( kontakt->stored_messages, temp_storedmsg->data);
-						
-              applet_update( Current_Status, FALSE );
-              if( g_slist_length( kontakt->stored_messages ) == 0 )
-                {
-                  kontakt->icon_p = get_pixmap_for_status( kontakt->status );
-                  kontakt->icon_b = get_bitmap_for_status( kontakt->status );
-                  kontakt->need_update = 1;
-                }
-						
-              Show_Quick_Status();
-            }
-        }
-					
-
-      /* This should be reimplemented with a "More Info" button */
-      /* OK_Box( _("Chat Request Refused"), message ); */
-      break;
-
     case ICQ_CMDxTCP_FILE:
       packet_print( packet, packet_length,
                     PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "FILE REQ CANCEL" );
 
-      for(xfer = pending_xfers;xfer;xfer = xfer->next) {
-        if ( ( ((XferInfo*)xfer->data)->remote_contact->uin == uin) &&
-             ( ((XferInfo*)xfer->data)->seq == sequence))
+      /*      for(xfer = kontakt->file_xfers;xfer;xfer = xfer->next) {
+        if (((XferInfo*)xfer->data)->seq == sequence)
+          break;
+      }
+
+      if (xfer)
+        ft_cancel_transfer(0, xfer->data);
+      */
+      break;
+
+      /* cancel 
+  packet_print( packet, packet_length,
+                    PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "FILE REQ CANCEL" );
+
+      for(xfer = kontakt->file_xfers;xfer;xfer = xfer->next) {
+        if (((XferInfo*)xfer->data)->seq == sequence)
           break;
       }
 
@@ -1485,6 +1413,8 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock, UIN_T uin)
 
       break;
 
+      */
+
     default:
       break;
     }
@@ -1527,7 +1457,7 @@ int TCPFinishConnection( GIOChannel *source, GIOCondition cond, gpointer data )
 
   sock = g_io_channel_unix_get_fd( source );
 
-  TCPSendHandShake(contact, 0, kontakt->sok);
+  TCPSendHandShake(kontakt, kontakt->direct_cookie, kontakt->sok);
 
   if (kontakt->used_version >= 6)
     kontakt->have_tcp_connection = OUTGOING1;
@@ -1548,7 +1478,7 @@ int TCPFinishConnection( GIOChannel *source, GIOCondition cond, gpointer data )
 int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
 {
   BYTE *packet;
-  guint bytes_read;
+  gsize bytes_read;
   int sock;
   GSList *contact;
 
@@ -1591,15 +1521,15 @@ int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
 
   if( kontakt->tcp_buf_len == 0 )
     {
-      g_io_channel_read( source, (gchar*)&kontakt->tcp_buf_len,
-                         2, &bytes_read );
+      g_io_channel_read_chars( source, (gchar*)&kontakt->tcp_buf_len,
+                         2, &bytes_read, NULL );
 
       if( !bytes_read )
         {
           kontakt->have_tcp_connection = NO;
           kontakt->sok = 0;
           close( sock );
-          g_io_channel_close( source );
+          g_io_channel_shutdown( source, FALSE, NULL);
           kontakt->gioc = 0;
           return FALSE;
         }
@@ -1608,8 +1538,8 @@ int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
         {
           bytes_read = 0;
           while( !bytes_read )
-            g_io_channel_read( source, (gchar*)&kontakt->tcp_buf_len + 1,
-                               1, &bytes_read );
+            g_io_channel_read_chars( source, (gchar*)&kontakt->tcp_buf_len + 1,
+                               1, &bytes_read, NULL );
         }
 
       kontakt->tcp_buf_len = GINT16_FROM_LE(kontakt->tcp_buf_len); 
@@ -1625,9 +1555,9 @@ int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
   if( packet == NULL )
     return TRUE;
 
-  g_io_channel_read( source, packet + kontakt->tcp_buf_read,
+  g_io_channel_read_chars( source, packet + kontakt->tcp_buf_read,
                      kontakt->tcp_buf_len - kontakt->tcp_buf_read,
-                     &bytes_read );
+                     &bytes_read, NULL );
 
   kontakt->tcp_buf_read += bytes_read;
   if( kontakt->tcp_buf_read != kontakt->tcp_buf_len )
@@ -1645,96 +1575,29 @@ int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
 
 static int TCPTimeout( tcp_message *m )
 {
-  gchar *error;
-  static int first = TRUE;
-
   GSList *contact;
 
 #ifdef TRACE_FUNCTION
   g_print( "TCPTimeout\n" );
 #endif
 
-  if( first )
-    {
-      first = FALSE;
-      return TRUE;
-    }
-
   contact = Find_User( m->uin );
   if( contact == NULL )
     return TRUE;
 
-  animate_off();
-
-  if( kontakt->force_override == TRUE )
-    {
-      if( m->text != NULL )
-        v7_sendmsg(m->uin, m->text);
-
-      g_free( m->data );
-      g_free( m->text );
-      g_free( m );
-
-      kontakt->tcp_msg_queue = g_slist_remove( kontakt->tcp_msg_queue, m );
-
-      return TRUE;
-    }
-
-  error = g_strdup_printf( _("Unable to send a message directly to %s."), kontakt->nick );
-  m->dialog = gnome_message_box_new( error, GNOME_MESSAGE_BOX_QUESTION,
-                                     _("Retry"), _("Send Through Server"),
-                                     GNOME_STOCK_BUTTON_CANCEL, NULL );
-  g_free( error );
-  gtk_window_set_title( GTK_WINDOW( m->dialog ), _("GnomeICU: Question") );
-  gtk_window_set_wmclass( GTK_WINDOW( m->dialog ), "Question", "GnomeICU" );
-  gnome_dialog_set_default( GNOME_DIALOG( m->dialog ), 0 );
-	
-  gtk_signal_connect( GTK_OBJECT( m->dialog ), "clicked",
-                      GTK_SIGNAL_FUNC( timeout_clicked ), m );
+  gnomeicu_spinner_stop();
 
+  m->dialog = gtk_message_dialog_new (GTK_WINDOW (MainData->window),
+                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK,
+                                     _("Unable to connect to %s."),
+                                     kontakt->nick);
   gtk_widget_show( m->dialog );
 
-  return FALSE;
+  return TRUE;
 }
 
-void timeout_clicked( GnomeDialog *dialog, gint button, tcp_message *m )
-{
-  GSList *contact;
-
-  /* We do this in case the user was removed before the message
-     was sent.. */
-  contact = Find_User( m->uin );
-  if( contact == NULL )
-    return;
-
-  switch (button) {
-  case 0: /* Retry */
-    m->timeout = gtk_timeout_add( 10000, (GtkFunction)TCPTimeout, m );
-    animate_on();
-    return;
-
-  case 1: /* Send through server */
-    if( m->text != NULL )
-      v7_sendmsg(m->uin, m->text) ;
-    /* icq_sendmsg( m->uin, m->text, TRUE ); */
-
-    /* Next time force message through server */
-    kontakt->force_override = TRUE;
-    break;
-
-  case 2: /* Cancel */
-    break;
-  }
-	
-  g_free( m->data );
-  g_free( m->text );
-  g_free( m );
-
-  kontakt->tcp_msg_queue = g_slist_remove( kontakt->tcp_msg_queue, m );
-}
-
-
-void TCPSendHandShake(GSList *contact, DWORD sessionid, int socket)
+void TCPSendHandShake(Contact_Member *contact, DWORD sessionid, int socket)
 {
   TLVstack *tlvs;
  
@@ -1744,18 +1607,18 @@ void TCPSendHandShake(GSList *contact, DWORD sessionid, int socket)
 
   tlvs = new_tlvstack("  ", 2);
   
-  switch (kontakt->used_version) {
+  switch (contact->used_version) {
   case 2:    
   case 3:
   case 4:
   case 5:
     add_nontlv(tlvs, "\xFF", 1);
-    add_nontlv_dw_le(tlvs, kontakt->used_version);
-    if (kontakt->used_version < 4)
+    add_nontlv_dw_le(tlvs, contact->used_version);
+    if (contact->used_version < 4)
       add_nontlv_dw_le(tlvs, our_port);
     else
       add_nontlv_w_le(tlvs, 0);
-    add_nontlv_dw_le(tlvs, our_info->uin);
+    add_nontlv_dw_le(tlvs, atoi (our_info->uin));
     add_nontlv_dw_le(tlvs, our_ip);
     add_nontlv_dw_le(tlvs, our_ip);
     add_nontlv(tlvs, "\x04", 1); /* Mode */
@@ -1763,15 +1626,14 @@ void TCPSendHandShake(GSList *contact, DWORD sessionid, int socket)
     break;
   case 6:
   case 7:
-    if (!sessionid)
-      sessionid = 15;
-
+  case 8:
+    
     add_nontlv(tlvs, "\xFF", 1);
-    add_nontlv_dw_le(tlvs, kontakt->used_version);
-    add_nontlv_dw_le(tlvs, kontakt->uin);
+    add_nontlv_dw_le(tlvs, contact->used_version);
+    add_nontlv_dw_le(tlvs, atoi (contact->uin));
     add_nontlv_w_le(tlvs, 0);
     add_nontlv_dw_le(tlvs, our_port);
-    add_nontlv_dw_le(tlvs, our_info->uin);
+    add_nontlv_dw_le(tlvs, atoi (our_info->uin));
     add_nontlv_dw_le(tlvs, our_ip);
     add_nontlv_dw_le(tlvs, our_ip);
     add_nontlv(tlvs, "\x04", 1); /* Mode */
@@ -1779,8 +1641,12 @@ void TCPSendHandShake(GSList *contact, DWORD sessionid, int socket)
     add_nontlv_dw_le(tlvs, sessionid); /* NEED SESSION ID */
     add_nontlv_dw_le(tlvs, 50);
     add_nontlv_dw_le(tlvs, 3);
-    if (kontakt->used_version == 7);
+    if (contact->used_version == 7);
     add_nontlv_dw_le(tlvs, 0);
+
+    break;
+  default:
+    g_warning("Unknown version used by contact for processing packet: %d\n", contact->used_version);
   }
 
  
@@ -1797,7 +1663,7 @@ void TCPSendHandShake(GSList *contact, DWORD sessionid, int socket)
   
 
 
-void TCPSendHandShakeAck(GSList *contact, int socket)
+void TCPSendHandShakeAck(int socket)
 {
   
 #ifdef TRACE_FUNCTION
@@ -1810,160 +1676,3 @@ void TCPSendHandShakeAck(GSList *contact, int socket)
                 PACKET_TYPE_TCP | PACKET_DIRECTION_SEND,
                 "Sending handshake ack" );
 }
-
-
-int TCPRefuseChat(int sock, ChatContact *ccontact)
-{
-  int returncode;
-  
-#ifdef TRACE_FUNCTION
-  g_print("TCPRefuseChat\n");
-#endif
-  returncode = TCPSendPacket(ccontact->contact->uin, ICQ_CMDxTCP_CANCEL,
-                             ICQ_CMDxTCP_CHAT, 0, 0,
-                             "\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
-                             11, "", ICQ_ACKxTCP_REFUSE, ccontact->seq, FALSE);
-
-  g_free(ccontact);
-    
-  return returncode;
-
-}
-
-
-int TCPAcceptChat(int sock, ChatContact *ccontact)
-{
-  
-  int returncode;
-  struct sockaddr_in my_addr;
-  gboolean have_tcp_port = FALSE;
-  gint cx;
-  TLVstack *tlvs;
-  int length = sizeof(struct sockaddr);
-
-
-#ifdef TRACE_FUNCTION
-  g_print("TCPAcceptChat\n");
-#endif
-
-  if (! ccontact->chatsession) {
-    ccontact->chatsession = g_malloc0(sizeof(ChatSession));
-    Sessions = g_slist_append(Sessions, ccontact->chatsession);
-  }
-
-  if (ccontact->chatsession->listen_socket == 0) {
-    ccontact->chatsession->listen_socket = socket(AF_INET, SOCK_STREAM, 0);
-    if (ccontact->chatsession->listen_socket <= 0)
-      return FALSE;
-         
-    for (cx = min_tcp_port; cx <= max_tcp_port; cx++) {
-      my_addr.sin_family = AF_INET;
-      my_addr.sin_port = g_htons(cx);
-      my_addr.sin_addr.s_addr = g_htonl(INADDR_ANY);
-      memset(&(my_addr.sin_zero), 0x00, 8);
-      if (bind
-          (ccontact->chatsession->listen_socket, (struct sockaddr *) &my_addr,
-           sizeof(struct sockaddr)) != -1)
-        have_tcp_port = TRUE;
-      else
-        continue;
-           
-      break;
-    }
-    
-    if (! have_tcp_port)
-      return FALSE;
-
-    
-    listen(ccontact->chatsession->listen_socket, 1);
-    ccontact->chatsession->gdk_input =
-      gdk_input_add(ccontact->chatsession->listen_socket, GDK_INPUT_READ,
-                    (GdkInputFunction) TCPChatHandshake, ccontact->chatsession);
-    
-    getsockname(ccontact->chatsession->listen_socket, (struct sockaddr *) &my_addr, &length);
-    
-    ccontact->chatsession->listen_port = g_ntohs(my_addr.sin_port);
-  }
-  
-  ccontact->chatsession->initiator = (void *) ccontact;
-  
-  tlvs = new_tlvstack(NULL, 0);
-
-  add_nontlv_lnts(tlvs, "");
-  add_nontlv_dw_be(tlvs, ccontact->chatsession->listen_port);
-  add_nontlv_dw_le(tlvs, ccontact->chatsession->listen_port);
-
-  returncode = TCPAckPacket(ccontact->contact->uin, ICQ_CMDxTCP_CHAT,
-                            tlvs->beg, tlvs->len, ICQ_ACKxTCP_ONLINE,
-                            ccontact->seq);
-
-  free_tlvstack(tlvs);
-
-  if (returncode && ccontact->mpartynew > 1)
-    TCPConnectChat(ccontact->listen_port, ccontact->contact->uin, ccontact->chatsession);
-  
-  return returncode;
-}
-
-int TCPSendChatRequest(UIN_T uin, gchar * msg, ChatSession *csession)
-{
-  GSList *temp_ccontact;
-  TLVstack *tlvs;
-  GSList *contact;
-  GString *sessionname = NULL;
-  int returncode;
-  
-#ifdef TRACE_FUNCTION
-  g_print("TCPSendChatRequest\n");
-#endif
-  
-  contact = Find_User( uin );
-  if (contact == NULL)
-    return 0;
-  
-  if (kontakt->has_direct_connect != YES)
-    return 0;
-  
-    
-  if (csession == NULL) {
-    csession = g_malloc0(sizeof(ChatSession));
-    Sessions = g_slist_append(Sessions, csession);
-  }
-  
-  kontakt->chat_requests = g_list_append(kontakt->chat_requests, csession);
-  
-  rus_conv(RUS_KOI_WIN, msg);
-
-  tlvs = new_tlvstack(NULL, 0);
-  
-  if (csession->members && g_slist_length(csession->members) > 0) {
-    
-    sessionname = g_string_new(our_info->nick);
-    
-    for (temp_ccontact = csession->members;
-         temp_ccontact != NULL;
-         temp_ccontact = g_slist_next(temp_ccontact)) {
-      g_string_sprintfa(sessionname, ",%s", ((ChatContact *)temp_ccontact->data)->contact->nick);
-    }
-    g_assert (sessionname != NULL);
-    add_nontlv_lnts(tlvs, sessionname->str);
-    add_nontlv_dw_be(tlvs, csession->listen_port);
-    add_nontlv_dw_le(tlvs, csession->listen_port);
-    
-    g_string_free (sessionname, TRUE);
-  }
-  else {
-    add_nontlv_lnts(tlvs, "");
-    add_nontlv_dw_be(tlvs, 0);
-    add_nontlv_dw_le(tlvs, 0);
-  }
-  
-  returncode = TCPSendPacket(uin, ICQ_CMDxTCP_START, ICQ_CMDxTCP_CHAT,
-                             0x10, 0, tlvs->beg, tlvs->len, msg, 0, 0, TRUE);
-
-  free_tlvstack(tlvs);
-  
-  return returncode;
-}
-
-
