@@ -26,15 +26,18 @@
  *
  * <?xml?>
  * <gnomeicu xmlns="http://gnomeicu.sourceforge.net/xmlcontact">
- *   <version>0.92</version>
+ *   <version>0.94.1</version>
  *   <icq>
  *     <uin>11111111</uin>
+ *     <password>mypasswd</password>
  *     <nick>my nick</nick>
+ *     <require-auth>true</require-auth>
+ *     <webpresence>true</webpresence>
  *     <timeStamp>10234354</timeStamp>
  *     <totalRecord>11</totalRecord>
  *     <status uid="222">normal</status>
  *     <group name="General" gid="111"/>
- *     ...
+ *     ... repeat <group/>
  *     <contacts>
  *       <user gid="222" uid="2234">
  *          <uin>34335333</uin>
@@ -46,12 +49,13 @@
  *          <notify/> <!-- online notification, gnomeicu client side feature -->
  *          <uin>23434343</uin>
  *          <nick>foo</nick>
+ *          <lastseen>321313</lastseen> <!-- time_t when last seen (at UTC) -->
  *       </user>
  *       <user> <!-- anonymous users, usually on your ignore list -->
  *          <status uid="3454">ignore</status>
  *          <uin>23565675</uin>
  *       </user>
- *       ...
+ *       ... repeat <user/>
  *     </contacts>
  *   </icq>
  * </gnomeicu>
@@ -59,6 +63,10 @@
 
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
+#include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <stdio.h>
 
 #include "common.h"
 #include "gnomeicu.h"
@@ -66,8 +74,9 @@
 #include "response.h"
 #include "v7send.h"
 #include "v7snac13.h"
+#include "util.h"
 
-#define CONTACT_FILE_VERSION "0.92"
+#define CONTACT_FILE_VERSION "0.94.1"
 
 static void xmlcontact_save_icq (xmlNodePtr icuNode);
 static void icq_contact_set_status (xmlNodePtr node, const gchar *status,
@@ -81,7 +90,10 @@ typedef enum {
   PARSER_VERSION,
   PARSER_ICQ,
   PARSER_ICQ_UIN,
+  PARSER_ICQ_PASSWORD,
   PARSER_ICQ_NICK,
+  PARSER_ICQ_REQUIRE_AUTH,
+  PARSER_ICQ_WEB_PRESENCE,
   PARSER_ICQ_TIMESTAMP,
   PARSER_ICQ_TOTALRECORD,
   PARSER_ICQ_STATUS,
@@ -90,6 +102,7 @@ typedef enum {
   PARSER_ICQ_USER,
   PARSER_ICQ_WAIT_AUTH,
   PARSER_ICQ_NOTIFY,
+  PARSER_ICQ_LAST_SEEN,
   PARSER_UNKNOWN
 } ParserState;
 
@@ -127,12 +140,13 @@ struct _XMLContactParseState {
 
   gboolean owner; /* switch between owner or user mode during parsing */
 
-  gint c_uin;
+  UIN_T c_uin;
   gchar *c_nick;
   gint c_temp_status_id;
   gint c_vis_id, c_inv_id, c_ign_id;
   gboolean wait_auth;
   gboolean notify;
+  time_t last_seen;
 };
 
 /* initialize the state */
@@ -169,8 +183,8 @@ xmlContactEndDoc (XMLContactParseState *state)
 
 /* called when encountered start of an element, also passed attributes, if any */
 static void
-xmlContactStartEle (XMLContactParseState *state, const CHAR *name,
-                    const CHAR **atts)
+xmlContactStartEle (XMLContactParseState *state, const xmlChar *name,
+                    const xmlChar **atts)
 {
   switch (state->state) {
   case PARSER_START:
@@ -237,6 +251,8 @@ xmlContactStartEle (XMLContactParseState *state, const CHAR *name,
       state->state = PARSER_ICQ_TIMESTAMP;
     } else if (!strcmp (name, "totalRecord")) {
       state->state = PARSER_ICQ_TOTALRECORD;
+    } else if (!strcmp (name, "password")) {
+      state->state = PARSER_ICQ_PASSWORD;
     } else {
       /* unknown element */
       state->prev_state = state->state;
@@ -252,6 +268,9 @@ xmlContactStartEle (XMLContactParseState *state, const CHAR *name,
   case PARSER_ICQ_GROUP:
   case PARSER_ICQ_WAIT_AUTH:
   case PARSER_ICQ_NOTIFY:
+  case PARSER_ICQ_PASSWORD:
+  case PARSER_ICQ_WEB_PRESENCE:
+  case PARSER_ICQ_LAST_SEEN:
     /* should have no element after these states */
     state->prev_state = state->state;
     state->state = PARSER_UNKNOWN;
@@ -260,6 +279,7 @@ xmlContactStartEle (XMLContactParseState *state, const CHAR *name,
   case PARSER_ICQ_CONTACT: /* inside <contact> */
     if (!strcmp (name, "user")) {
       state->state = PARSER_ICQ_USER;
+
       state->gid = 0;
       state->uid = 0;
       /* read the user gid and uid */
@@ -282,6 +302,8 @@ xmlContactStartEle (XMLContactParseState *state, const CHAR *name,
       state->state = PARSER_ICQ_UIN;
     } else if (!strcmp (name, "nick")) {
       state->state = PARSER_ICQ_NICK;
+    } else if (!strcmp (name, "lastseen")) {
+      state->state = PARSER_ICQ_LAST_SEEN;
     } else if (!strcmp (name, "status")) {
       state->state = PARSER_ICQ_STATUS;
       /* parse status uid */
@@ -311,9 +333,11 @@ xmlContactStartEle (XMLContactParseState *state, const CHAR *name,
 }
 
 static void
-xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
+xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
 {
   CONTACT_PTR contactdata;
+  GSList *contact;
+  gint len;
 
   switch (state->state) {
   case PARSER_UNKNOWN:
@@ -335,6 +359,7 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
     /* compare the version string */
     if (strcmp (state->content->str, CONTACT_FILE_VERSION)) {
       /* there is version difference, force re-sync contacts list */
+/*      printf ("see version difference\n");*/
       contacts_resync = TRUE;
     }
     g_string_truncate (state->content, 0);
@@ -345,19 +370,55 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
   case PARSER_ICQ_UIN:
     if (state->owner) {
       state->state = PARSER_ICQ;
+      if (our_info->uin == NULL) /* in case we got it in newsignup.c */
+        our_info->uin = g_strdup (state->content->str);
     } else {
       state->state = PARSER_ICQ_USER;
-      state->c_uin = atoi (state->content->str);
+      state->c_uin = g_strdup (state->content->str);
     }
     g_string_truncate (state->content, 0);
     break;
+  case PARSER_ICQ_PASSWORD:
+    state->state = PARSER_ICQ;
+    if (passwd) {
+      g_free (passwd);
+    }
+    passwd = g_strdup (state->content->str);
+    g_string_truncate (state->content, 0);
+    break;
   case PARSER_ICQ_NICK:
+    len = strlen (state->content->str);
     if (state->owner) {
       state->state = PARSER_ICQ;
       /*g_message ("owner nick is: %s", state->content->str);*/
+      our_info->nick = g_markup_escape_text (state->content->str, len);
     } else {
       state->state = PARSER_ICQ_USER;
-      state->c_nick = g_strdup (state->content->str);
+      state->c_nick = g_markup_escape_text (state->content->str, len);
+    }
+    g_string_truncate (state->content, 0);
+    break;
+  case PARSER_ICQ_LAST_SEEN:
+    if (state->content->len)
+      state->last_seen = strtoul(state->content->str, NULL, 10);
+    state->state = PARSER_ICQ_USER;
+    g_string_truncate (state->content, 0);
+    break;
+  case PARSER_ICQ_REQUIRE_AUTH:
+    state->state = PARSER_ICQ;
+    if (!strcmp (state->content->str, "true")) {
+      our_info->auth = TRUE;
+    } else if (!strcmp (state->content->str, "false")) {
+      our_info->auth = FALSE;
+    }
+    g_string_truncate (state->content, 0);
+    break;
+  case PARSER_ICQ_WEB_PRESENCE:
+    state->state = PARSER_ICQ;
+    if (!strcmp (state->content->str, "true")) {
+      our_info->webpresence = TRUE;
+    } else if (!strcmp (state->content->str, "false")) {
+      our_info->webpresence = FALSE;
     }
     g_string_truncate (state->content, 0);
     break;
@@ -409,11 +470,14 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
 
     contactdata = g_new0 (Contact_Member, 1);
 
-    contactdata->uin = state->c_uin;
-    if (state->c_nick)
-      g_snprintf (contactdata->nick, 20, "%s", state->c_nick);
-    else
-      g_snprintf (contactdata->nick, 20, "%d", state->c_uin);
+    contactdata->uin = g_strdup (state->c_uin);
+    if (state->c_nick) {
+		g_free(contactdata->nick);
+		contactdata->nick = g_strdup(state->c_nick);
+	  } else {
+		g_free(contactdata->nick);
+		contactdata->nick = g_strdup(state->c_uin);
+	  }
 
     contactdata->gid = state->gid;
     contactdata->uid = state->uid;
@@ -434,7 +498,6 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
     contactdata->status = STATUS_OFFLINE;
     contactdata->last_time = -1L;
     contactdata->current_ip = -1L;
-    contactdata->show_again = TRUE;
     contactdata->info = g_new0 (USER_INFO_STRUCT, 1);
     ((USER_INFO_STRUCT *)contactdata->info)->sex = -1;
     contactdata->inlist = TRUE;
@@ -442,9 +505,18 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
     contactdata->confirmed = TRUE;
     contactdata->wait_auth = state->wait_auth;
     contactdata->online_notify = state->notify;
+    contactdata->last_seen = state->last_seen;
 
+    contact = Find_User (contactdata->uin);
+    while (contact && kontakt->gid == 0) {
+      Contacts = g_slist_remove (Contacts, contact->data);
+      contact = Find_User (contactdata->uin);
+    }
     Contacts = g_slist_append (Contacts, contactdata);
 
+	g_free (state->c_uin);
+	state->c_uin = NULL;
+
     if (state->c_nick) {
       g_free (state->c_nick);
       state->c_nick = NULL;
@@ -454,6 +526,7 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
     state->c_ign_id = 0;
     state->wait_auth = FALSE;
     state->notify = FALSE;
+    state->last_seen = 0;
 
     break;
   case PARSER_ICQ_WAIT_AUTH:
@@ -465,17 +538,21 @@ xmlContactEndEle (XMLContactParseState *state, const CHAR *name)
 
 /* read the CDATA inside each <element></element> */
 static void
-xmlContactChars (XMLContactParseState *state, const CHAR *chars, gint len)
+xmlContactChars (XMLContactParseState *state, const xmlChar *chars, gint len)
 {
   gint i;
 
   if (state->state == PARSER_VERSION ||
       state->state == PARSER_ICQ_UIN ||
+      state->state == PARSER_ICQ_PASSWORD ||
       state->state == PARSER_ICQ_NICK ||
+      state->state == PARSER_ICQ_REQUIRE_AUTH ||
+      state->state == PARSER_ICQ_WEB_PRESENCE ||
       state->state == PARSER_ICQ_STATUS ||
       state->state == PARSER_ICQ_GROUP ||
       state->state == PARSER_ICQ_TIMESTAMP ||
-      state->state == PARSER_ICQ_TOTALRECORD)
+      state->state == PARSER_ICQ_TOTALRECORD ||
+      state->state == PARSER_ICQ_LAST_SEEN)
     for (i = 0; i < len; i++)
       g_string_append_c (state->content, chars[i]);
 }
@@ -543,10 +620,14 @@ xmlcontact_load (const gchar *filename)
   xmlParserCtxtPtr ctxt;
   XMLContactParseState state;
 
+  state.content=NULL;
+
   ctxt = xmlCreateFileParserCtxt (filename);
 
-  if (!ctxt)
+  if (!ctxt) {
+    contacts_resync = TRUE;
     return FALSE;
+  }
 
   srvlist_exist = TRUE;
 
@@ -567,6 +648,7 @@ xmlcontact_save (const gchar *filename)
   xmlDocPtr doc;
   xmlNsPtr icuNs;
   xmlNodePtr node;
+  gchar *tempfilename;
 
   doc = xmlNewDoc ("1.0");
 
@@ -580,9 +662,18 @@ xmlcontact_save (const gchar *filename)
   /* save icq contacts */
   xmlcontact_save_icq (doc->xmlRootNode);
 
-  if (xmlSaveFile (filename, doc) == -1) {
-    g_warning ("Cannot save XML contact file: %s", filename);
+  tempfilename =  g_strdup_printf("%s.tmp",filename);
+
+  if (xmlSaveFile (filename, doc) <= 0) {
+    g_warning ("Cannot save XML contact file: %s", tempfilename);
   }
+  else if (!rename (tempfilename, filename))
+    g_warning("Cannot save XML contacts into %s, but saved it in %s", filename,
+	      tempfilename);
+  
+  g_free(tempfilename);
+
+  chmod(filename, S_IRUSR|S_IWUSR);
 
   xmlFreeDoc (doc);
 }
@@ -598,14 +689,24 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
   gchar *tmpstr;
   GSList *tlist, *contact;
   gint contact_count = 0;
-
+  xmlChar *xmlEncoded;
+	
   icqNode = xmlNewChild (icuNode, NULL, "icq", NULL);
 
-  tmpstr = g_strdup_printf ("%d", our_info->uin);
-  xmlNewChild (icqNode, NULL, "uin", tmpstr);
-  g_free (tmpstr);
+  xmlNewChild (icqNode, NULL, "uin", our_info->uin);
+	
+  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, passwd);
+  xmlNewChild (icqNode, NULL, "password", xmlEncoded);
+  free (xmlEncoded);
+  xmlEncoded = NULL;
+	
+  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, our_info->nick);
+  xmlNewChild (icqNode, NULL, "nick", xmlEncoded);
+  free (xmlEncoded);
+  xmlEncoded = NULL;
 
-  xmlNewChild (icqNode, NULL, "nick", our_info->nick);
+  xmlNewChild (icqNode, NULL, "require-auth", our_info->auth ? "true" : "false");
+  xmlNewChild (icqNode, NULL, "webpresence", our_info->webpresence ? "true" : "false");
 
   tmpstr = g_strdup_printf ("%d", list_time_stamp);
   xmlNewChild (icqNode, NULL, "timeStamp", tmpstr);
@@ -615,7 +716,6 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
   xmlNewChild (icqNode, NULL, "totalRecord", tmpstr);
   g_free (tmpstr);
 
-  /* FIXME: status element should not contain the string "status" */
   tmpstr = g_strdup_printf ("%d", Current_Status == STATUS_OFFLINE ?
                             STATUS_ONLINE : Current_Status);
   child = xmlNewChild (icqNode, NULL, "status", tmpstr);
@@ -643,7 +743,11 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
   if (contact)
     contactNode = xmlNewChild (icqNode, NULL, "contacts", NULL);
 
-  while (contact) {
+  for (; contact ; contact = contact->next) {
+    
+    if (!kontakt->inlist)
+      continue;
+
     contact_count++;
 
     userNode = xmlNewChild (contactNode, NULL, "user", NULL);
@@ -674,16 +778,14 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
     }
 
     /* UIN */
-    tmpstr = g_strdup_printf ("%d", kontakt->uin);
+    tmpstr = g_strdup (kontakt->uin);
     xmlNewChild (userNode, NULL, "uin", tmpstr);
     g_free (tmpstr);
 
     /* skip the users that are only on one of the status list,
        because they don't have gid/uid/nick saved on the contact list */
-    if (!kontakt->uid) {
-      contact = contact->next;
+    if (!kontakt->uid)
       continue;
-    }
 
     /* gid */
     tmpstr = g_strdup_printf ("%d", kontakt->gid);
@@ -698,7 +800,13 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
     /* nick */
     xmlNewChild (userNode, NULL, "nick", kontakt->nick);
 
-    contact = contact->next;
+    /* Last seen online */
+    if (kontakt->status != STATUS_OFFLINE)
+      kontakt->last_seen = time(NULL);
+    tmpstr = g_strdup_printf("%ld", kontakt->last_seen);
+    xmlNewChild (userNode, NULL, "lastseen", tmpstr);
+    g_free (tmpstr);
+
   }
 }
 
